# encoding: utf-8

from xmmsapi import \
		method_arg, \
		method_varg, \
		service_method, \
		service_broadcast, \
		service_constant, \
		XmmsServiceNamespace, \
		client_method, \
		client_broadcast, \
		XmmsServiceClient
